/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/


/*******************************************************************************
 *
 *  main.c - c file for main function
 *  DRV8833C_FIRMWARE
 *  10/24/2014
 *
 ******************************************************************************/

/* 				Program Structure
 *
 *					 main.c
 *					  ^^^^
 *
 */

#include <msp430g2553.h>

#define STEP_DIR_OUT    (P1DIR |= BIT4 + BIT3 + BIT2 + BIT1 + BIT0)          // Set PWM output direction of P1.0-1.3
#define STEP_ALL_OFF    (P1OUT &= ~(BIT3 + BIT2 + BIT1 + BIT0))
#define BUTTON_DIR_IN   (P2DIR &= ~(BIT4 + BIT3 + BIT2 + BIT1 + BIT0))       //Set button input direction of P2.0-2.3
#define ENABLE_PULL     (P2REN |= BIT4 + BIT3 + BIT2 + BIT1 + BIT0)
#define PULL_UP         (P2OUT |= BIT4 + BIT3 + BIT2 + BIT1 + BIT0)
#define AIN1_LOW        (P1OUT &= ~BIT0)
#define AIN1_HIGH       (P1OUT |= BIT0)
#define AIN2_LOW        (P1OUT &= ~BIT1)
#define AIN2_HIGH       (P1OUT |= BIT1)
#define BIN2_LOW        (P1OUT &= ~BIT2)
#define BIN2_HIGH       (P1OUT |= BIT2)
#define BIN1_LOW        (P1OUT &= ~BIT3)
#define BIN1_HIGH       (P1OUT |= BIT3)

#define START_STOP      (P2IN & BIT0)
#define ACCEL           (P2IN & BIT1)
#define DECEL			(P2IN & BIT2)
#define DIR				(P2IN & BIT3)
#define FULL_HALF       (P2IN & BIT4)

unsigned int Accel_Rate = 200;                    //Acceleration rate 200pps/s
unsigned int StartSpeed = 100;                    //Starting speed
unsigned int SpeedRate;
unsigned int temp_Accel = 1;
unsigned int temp_Decel = 1;
unsigned int temp_Full = 1;
unsigned int temp_Half = 1;
unsigned int temp_STOP = 0;
unsigned int temp_Accel_Rate;
int StepLogic = 0;
unsigned int Speed_Full;
unsigned int Speed_Half;
unsigned int temp_Start = 1;
unsigned int i;
unsigned int Motor_Start = 0;
unsigned int Count = 0;
unsigned int wdt_count = 0;


unsigned int PPS2Timer (unsigned int PPSValue)
{
	int TimerValue;
	TimerValue = 1000000/PPSValue;
	return TimerValue;
}

void StepMotor_Init()                             //Initialization
{
	STEP_DIR_OUT;
	STEP_ALL_OFF;
	BUTTON_DIR_IN;
	ENABLE_PULL;
	PULL_UP;
	DCOCTL  = CALDCO_8MHZ;			          // Set DCO to 8MHz
	BCSCTL1 = CALBC1_8MHZ | XTS;
	BCSCTL3 = LFXT1S_2;			              // ACLK = VLO
	BCSCTL2 = DIVS_3;                         // set SMCLK 8MHz/8=1MHz
	TA0CTL = TASSEL_2 + MC_1 + ID_0;          // Timer0 clk configuration
	TA1CTL = TASSEL_2 + MC_1 + ID_0;          // Timer1 clk configuration
	SpeedRate = StartSpeed;

	P2DIR |=BIT5;
	WDTCTL = WDT_MDLY_0_5 ;                    // Set Watchdog Timer interval to ~0.5ms
	IE1 |= WDTIE;                             // Enable WDT interrupt

}

void Accel_Decel()
{
	//Acceleration
	if ((( ACCEL == 0)&&(DECEL == BIT2))&& temp_Accel)
	{
		temp_Accel = 0;
		TA1CCTL0 |= CCIE;                           // TimerA1 interrupt enabled
		TA1CCR0 = 1000000/temp_Accel_Rate;          // Interrupt period
	}
	if (ACCEL == BIT1)
	{
		temp_Accel = 1;
	}
	//Deceleration
	if((DECEL == 0)&& (ACCEL ==BIT1)&& temp_Decel)
	{
		temp_Decel = 0;
		TA1CCTL0 |= CCIE;                           // TimerA1 interrupt enabled
		TA1CCR0 = 1000000/temp_Accel_Rate;          // Interrupt period
	}
	if (DECEL == BIT2)
	{
		temp_Decel = 1;
	}
	// TimerA1 interrupt disabled
	if(((DECEL ==BIT2 )&&(ACCEL == BIT1))||((DECEL == 0 )&&(ACCEL == 0)))
	{
		TA1CCTL0 &= ~CCIE;
		temp_Decel = 1;
		temp_Accel = 1;
	}
}

// full step routine
void Full_Step()
{
	if((P2IN & BIT3) == BIT3)                     //Forward
	{
		StepLogic++;
		if(StepLogic > 3) StepLogic = 0;
	}
	else                                        // Reverse
	{
		StepLogic--;
		if(StepLogic < 0) StepLogic = 3;
	}
	switch (StepLogic)                          //Stepper waveform logic
	{
	case 0:										// A+, B+
		AIN1_HIGH;
		AIN2_LOW;
		BIN1_HIGH;
		BIN2_LOW;
		break;
	case 1:										// A-, B+
		AIN1_LOW;
		AIN2_HIGH;
		BIN1_HIGH;
		BIN2_LOW;
		break;
	case 2:										// A-, B-
		AIN1_LOW;
		AIN2_HIGH;
		BIN1_LOW;
		BIN2_HIGH;
		break;
	case 3:										// A+, B-
		AIN1_HIGH;
		AIN2_LOW;
		BIN1_LOW;
		BIN2_HIGH;
		break;
	}
}

// half step routine
void Half_Step()
{
	if((P2IN & BIT3) == BIT3)                   //Forward
	{
		StepLogic--;
		if(StepLogic < 0) StepLogic = 7;
	}
	else                                        // Reverse
	{
		StepLogic++;
		if(StepLogic > 7) StepLogic = 0;
	}
	switch (StepLogic)                          //Stepper waveform logic
	{
	case 0:										// A off, B+
		AIN1_LOW;
		AIN2_LOW;
		BIN1_HIGH;
		BIN2_LOW;
		break;
	case 1:										// A+, B+
		AIN1_HIGH;
		AIN2_LOW;
		BIN1_HIGH;
		BIN2_LOW;
		break;
	case 2:										// A+, B off
		AIN1_HIGH;
		AIN2_LOW;
		BIN1_LOW;
		BIN2_LOW;
		break;
	case 3:										// A+, B-
		AIN1_HIGH;
		AIN2_LOW;
		BIN1_LOW;
		BIN2_HIGH;
		break;
	case 4:										// A off, B-
		AIN1_LOW;
		AIN2_LOW;
		BIN1_LOW;
		BIN2_HIGH;
		break;
	case 5:										// A-, B-
		AIN1_LOW;
		AIN2_HIGH;
		BIN1_LOW;
		BIN2_HIGH;
		break;
	case 6:										// A-, B off
		AIN1_LOW;
		AIN2_HIGH;
		BIN1_LOW;
		BIN2_LOW;
		break;
	case 7:										// A-, B+
		AIN1_LOW;
		AIN2_HIGH;
		BIN1_HIGH;
		BIN2_LOW;
		break;
	}
}


// main routine
//		initializes ports
//		reads inputs switches to start/stop
//					 increase/decrease speed
void main(void)
{
	_enable_interrupts();
	StepMotor_Init();
	while (1)
	{
		if((Motor_Start == 1)&& temp_Start)
		{
			temp_Start=0;
			temp_Full = 1;
			temp_Half = 1;
			temp_STOP = 1;
			SpeedRate = StartSpeed;
		}
		if(Motor_Start == 1)                                //Motor start
		{
			if((FULL_HALF== 0 )&&temp_Full)                 //Full step
			{
				temp_Full = 0;
				temp_Half = 1;
				Speed_Full = SpeedRate;
				TA0CCR0 = PPS2Timer(Speed_Full);            // TimerA0 Interrupt period
				temp_Accel_Rate = Accel_Rate;
				TA0CCTL0 |= CCIE;                           // TimerA0 interrupt enabled
			}
			if ((FULL_HALF == BIT4)&&temp_Half)             // Half step
			{
				temp_Half = 0;
				temp_Full = 1;
				Speed_Half = (SpeedRate<<1);                // Speed *2
				TA0CCR0 = PPS2Timer(Speed_Half);            // TimerA0 interrupt period
				temp_Accel_Rate = ( Accel_Rate<<1);         // ACCEL *2
				TA0CCTL0 |= CCIE;						   	// TimerA0 interrupt enabled
			}
			Accel_Decel();
		}
		if (Motor_Start == 0 && temp_STOP == 1)
		{
			temp_Start = 1;
			TA1CCTL0 |= CCIE;
			if(FULL_HALF == BIT4)
			{
				temp_Accel_Rate = (Accel_Rate << 1);
			}
			else temp_Accel_Rate = Accel_Rate;
			TA1CCR0 = 1000000/temp_Accel_Rate;          // Interrupt period
			if (SpeedRate < 45)                         // Set the stop speeds
			{
				STEP_ALL_OFF;
				TA1CCTL0 &= ~CCIE;
				TA0CCTL0 &= ~CCIE;
				temp_STOP = 0;
			}
		}
	} // end of while loop
} // end of main

#pragma vector = TIMER0_A0_VECTOR                //Stepper logic generate
__interrupt void TA0_ISR(void)
{
	if(FULL_HALF == 0)                           //Full step
	{
		Full_Step();
	}
	else
	{
		Half_Step();
	}
}

// Acceleration and deceleration interrupt
#pragma vector = TIMER1_A0_VECTOR
__interrupt void TA1_ISR(void)
{
	if(( ACCEL == 0)&&(DECEL == BIT2))
	{
		SpeedRate++;
		if(FULL_HALF == 0)
		{
			if(SpeedRate > 1000)                 // Up limit set for full step
			{
				SpeedRate = 1000;
			}
		}
		else
		{
			if(SpeedRate>2000 )
			{
				SpeedRate = 2000;				 // Up limit set for half step
			}
		}
	}
	if((DECEL == 0)&& (ACCEL ==BIT1))
	{
		SpeedRate--;
		if(FULL_HALF == 0)
		{
			if(SpeedRate < 100)                  // Down limit set for full step
			{
				SpeedRate = 100;
			}
		}
		else
		{
			if(SpeedRate < 200 )
			{
				SpeedRate = 200;			 	// Down limit set for half step
			}
		}
	}
	if(Motor_Start == 0 && temp_STOP == 1)		// Disable motor
	{
		SpeedRate--;
	}
	TA0CCR0 = PPS2Timer(SpeedRate);             // Interrupt period update
} // end of TIMER1_A0 interrupt

// Watchdog Timer interrupt service routine
#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer(void)
{
	wdt_count++;
	if(wdt_count>300)
	{
		P2OUT ^= BIT5;                          // 150ms blinking
		wdt_count=0;
	}
	if((P2IN & BIT0) == 0)					    // start/stop button filter
	{
		Count++;
		if (Count == 200)
		{
			if(Motor_Start == 0)
			{
				Motor_Start = 1;
			}
			else
			{
				Motor_Start = 0;
			}
		}
	}
	else
	{
		Count = 0;
	}
} // end of WDT_VECTOR interrupt
